%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Algorithme EMD en 2D
% Rfrence principale : Huang et aL. (1998)
%
% Auteur: Christophe Damerval 
% Laboratoire LMC-IMAG (Grenoble - France)
% 
% Date de cration: 20/10/2004
% 
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Schma d'appel des fonctions
% emd2d
%   -> calculIMFs
%     -> CalculEnveloppes
%        -> grilleInterpolation
%        -> interpolation
%        -> CritereArretSifting
%     -> CritereArretEMD
%   -> affichage
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Imodes,Ires,transformee]=emd2d(I, ...
			     messages,typeAffichage,...
			     methode,noModeMax,noIterMax,...
			     seuilExtrema,espacement)
% 
% On obtient K modes
% Imodes:  dimension (n,n,K)
% Iresidu: 
% transformee: transforme de Fourier des modes, dimension (n,n,K)
% 
% 
% I: image  traiter 
%  
% 
% 
% messages: affichage ou non des messages  ('true','false')
% 
% methode: type d'interpolation ('thin-plate','cubic')
%
% affichage: type d'affichage des modes ('image','signal')
%
% noModeMax: nombre maximum de modes  obtenir
% noIterMax: nombre maximum d'itrations  effectuer dans le sifting process
%
%
% espacement: espacement des points 
%   si espacement=length(I), on ne prend que les coins


if nargin==1
  methode='cubic';
  noModeMax=10;
  noIterMax=3;
  messages=true;
  typeAffichage='image+fourier';
  seuilExtrema=length(I)/8;
end

% on traite des rels
I=double(I);

if (messages)
  disp('EMD 2D (Tof)')
  disp(['Taille: ' num2str(size(I))])
  disp(['Methode: ' methode])
  disp(['Nb Modes Max: ' num2str(noModeMax)])
  disp(['Nb Iterations Maximum par mode: ' num2str(noIterMax)])
  disp(['Nombre d extrema mini pour extraire une IMF: ' num2str(seuilExtrema)])
end


% Calcul des IMFs (par l'algorithme EMD)
[Imodes,Ires]=calculIMFs(I,methode,noModeMax,noIterMax,messages,seuilExtrema);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Dimensions des rsultats

taille=length(I);

sz=size(Imodes);
if (length(sz)>=3)
  nbmodes=sz(3);
else
  nbmodes=1;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% transforme de Fourier


transformee=zeros(taille,taille,nbmodes+1);

for j=1:nbmodes
  transformee(:,:,j)=abs(fftshift(fft2(Imodes(:,:,j))))/taille^2;
end
transformee(:,:,nbmodes+1)=abs(fftshift(fft2(Ires)))/taille^2;

% Synthse 
if not(messages)
  return;
end

affichage(I,Imodes,Ires,transformee,nbmodes,typeAffichage);
